<?php

# This PHP script starts a short living sagent process which connects to a
# long living simv server (possibly started by bash script  simv_info_server ).
# It performs a simv command. As a reply it obtains temporary copies of the
# current file and possible duplicates which it displays in a HTML page.
# That page provides some links for navigating in the item list of simv
# and uses the query variables below to communicate with its next incarnation.

# The script can handle two query variables: 
#   simvidx    the list index of simv where the following actions are perforemed
#   simvcmd    the keyword for a simv action to be performed
#              (see function append_query_cmd() )
# both may be ommitted though. Then the current list item of simv is used
# resp. no action is performed.

# For an overview of the procedure see "main" area at the very end of this text



# --- Adjustable program parameters 

# The installation directory of stic (e.g /u/thomas/stic-0.2)
  $stic_dir= "INSERT_YOUR_STIC_DIRECTORY_HERE";

# The address of the simv server to connect with
  $simv_host= "localhost";
  $simv_port= 4000;


# The directory where to find the program sagent.
# Needs to have leading slash if not empty.
  $sagent_dir= "$stic_dir/bin/";

# Encryption is used if this variable is not set to "off"
  $simv_encryption= "off";

# The user id for encrypted connections
  $simv_user= "webserver";
# Where to find the keyfile webserver.tnl  (*must not* be reachable by URL)
  $simv_key_dir= "$stic_dir";


# If set to 1, the query variables, the call and the stdout of sagent are shown
  $verbous= 0;


# --- Public variables describing the current reply message 

  
# Format of parsed message in global array vmsg :
#  vmsg[header_subscr]  .... subscription type of message e.g. "current_file"
#  vmsg[header_fieldcount] . number of fields
#  vmsg[header_namelist] ... list of fieldnames
#  vmsg[$fieldname] ........ content of field with name $fieldname
  $vmsg["dummy"]= "dummy";

# Indicates wether a message has been completely received
  $vmsg_complete= false;




# --- Sending the request to the simv server


function append_query_cmd() {
# Here the query variable simvcmd is translated into a simv command.
# One should not directly forward its content because it can be easily
# manipulated at the client side.
  global $cmd;
  global $simvcmd;

  $cmdlen= strlen($cmd);
  if ($simvcmd == "back")
    $cmd .= "-back ";
  if($simvcmd == "refresh")
    $cmd .= "-refresh ";
  if ($simvcmd == "skip") 
    $cmd .= "-skip ";
  if (substr($simvcmd,0,4) == "jump" && strlen($simvcmd)>4) {
    if(substr($simvcmd,4,1) == " ")
      $jump_relative= "+";
    $msg= explode(" ",trim(substr($simvcmd,4,strlen($simvcmd))));
    if(!empty($msg[0]))
      $cmd .= "-jump:$jump_relative$msg[0]";
  }
  return 1;
}


function make_cmd() {
# Composes the start command of the sagent process which connects to the
# simv server. The command comprises of an initial part for sagent and
# some payload for simv.
#
  global $cmd;
  global $simv_port;
  global $simv_host;
  global $simvidx;
  global $simvcount;
  global $sagent_dir;
  global $simv_encryption;
  global $simv_user;
  global $simv_key_dir;

  # For a description and command reference of simv execute:  simv -help | less
  # This also documents all available sagent commands.

  $cmd  = "$sagent_dir" . "sagent ";

  # the initial arguments for sagent
  $cmd .= "-internal:all:off ";
  $cmd .= "-end_on:mark:done ";
  $cmd .= "-end_on:timeout:60 ";
  $cmd .= "-add_view:-:current_file_copy:default:encapsulated ";
  $cmd .= "-add_view:-:current_file:default ";
  $cmd .= "-add_view:-:double_files_copy:default ";
  $cmd .= "-add_view:-:double_files:default ";
  $cmd .= "-add_view:-:stdout:default ";

  if("$simv_encryption" != "off") {
    $cmd .= "-security:tcp:keydirectory:$simv_key_dir ";
    $cmd .= "-security:tcp:user:$simv_user ";
  } 

  $cmd .= "-arguments_to_queue ";
  $cmd .= "-tcp_client:$simv_host:$simv_port:default:encapsulated ";

  # these will be forwarded to simv
  $cmd .= "-bundle:start ";
  if(! empty($simvidx)) {
    # navigate to the last known position (in case other clients are around)
    $cmd .= "-jump:$simvidx ";
  }
  # try to find out what action the our client desires
  append_query_cmd();
  $cmd .= "-refresh ";
  $cmd .= "-request:current_item:-list:0:0:count ";
  $cmd .= "-mark:done: ";
  $cmd .= "-bundle:end ";

  return 1;
}


# --- receiving the reply from the simv server


# internal variables of the parser

  # current message
  $vmsg_stage= 0;
  # 0= not started , 1= reading name , 2= reading representation ,
  # 3= reading number_of_bytes , 4= reading bytes , -1= done
  $vmsg_buffer= "";
  $vmsg_nof= 0;
  $vmsg_count= 0;

  # current message field
  $vfield_stage= 0;
  # 0= not started , 1= reading type ,  2= reading number_of_fields
  # 3= reading fields , 4= reading linefeed , -1= done
  $vfield_nob= 0;
  $vfield_name= "";


function parse_vmsg($line) {
#
# caution: use this only with default representation of field data
#          this parser is not suitable for full 8-bit binary data
#
# Usage: Call this function for each complete line of reply text received
#        from sagent. (i.e buffer bytes until a newline character is received)
#        If $vmsg_complete = "true" then a reply message is parsed completely
#        and may be read from $vmsg. Else feed the next line.
# 
  global $vmsg;
  global $vmsg_complete;
  global $vmsg_stage;
  global $vmsg_nof;
  global $vmsg_fcount;
  global $vfield_stage;
  global $vfield_nob;
  global $vfield_name;
  global $vmsg_buffer;
  
  $got= "$line";
  if ($vmsg_stage <= 0) {
    # ???  unset vmsg
    $vmsg["header_subscr"]= "";
    $vmsg["header_fieldount"]= 0;
    $vmsg["header_namelist"]= "";
    $vmsg_nof= 0;
    $vmsg_fcount= 0;
    $vmsg_complete= false;
    $vfield_stage= 0;
    $tend= strpos($got," ");
    if ($tend == false) {
      return 0;
    } else {
      $vmsg["header_subscr"]=  substr($got,0,$tend);
      $got= substr($got, $tend+1);
    }
    $npt= strpos($got,"\n");
    $bpt= strpos($got," ");
    if ( $npt != false && ( $bpt == false || $npt < $bpt ) ) {
      $vmsg_buffer= substr($got, 0, $npt);
      $got= substr($got, $npt);
    } else {
      if ( $bpt != false ) {
        $vmsg_buffer= substr($got, 0, $bpt);
        $got= substr($got, $bpt+1);
      }
    }
    $vmsg_nof= $vmsg_buffer;
    $vmsg["header_fieldcount"]= $vmsg_nof;
    $vmsg_stage= 3;
  }
  if ($vmsg_stage == 3) {
    while ($vmsg_fcount < $vmsg_nof) {
      if ($vfield_stage<=0) {
        $ldpt= 1;
        $dpt= strpos(substr($got, $ldpt), ":");
        if ($dpt == -1) 
          return 0;
        $vfield_name=  substr($got, $ldpt, $dpt);
        $ldpt= $ldpt+$dpt+1;
        $dpt= strpos(substr($got, $ldpt), ":");
        if ($dpt == -1) 
          return 0;
        $ldpt= $ldpt+$dpt+1;
        $dpt= strpos(substr($got, $ldpt), ":");
        if ($dpt == -1) 
          return 0;
        $vfield_nob= substr($got, $ldpt, $dpt);
        $ldpt= $ldpt+$dpt+1;
        $got= substr($got, $ldpt);
        if ($vmsg["header_namelist"] != "")
          $vmsg["header_namelist"].= " ";
        $vmsg["header_namelist"].= $vfield_name;
        $vmsg["$vfield_name"]= "";
        $vfield_stage= 4;
      }
      if ($vfield_stage == 4) {
        $l= strlen($got);
        if ($l < $vfield_nob) {
          $vmsg["$vfield_name"].= $got;
          $vfield_nob= $vfield_nob-$l;
          return 2;
        } else {
          $vmsg["$vfield_name"].= substr($got, 0, $vfield_nob);
          $got= substr($got, $vfield_nob);
          $vmsg_fcount++;
          $vfield_stage= -1;
        }
      }
    }
    $vmsg_stage= 4;
  }
  if ($vmsg_stage == 4) {
    if ($got == "")
      return 2;
    $vmsg_stage= -1;
    $vmsg_complete= true;
  }
  return 1;
}


# internal variables of the reply interpreter

# state variable which tells what kind of -request is currently pending
  $reply_mode= "off";


function interpret_reply() {
# Receives all replies from sagent, parses them and sets the parameters for
# the HTML page to be generated. This function also detects when the
# transaction is completed and eventually ends.
#
  global $fp_in;
  global $vmsg,$vmsg_complete;
  global $reply_mode;
  global $stdout_buffer;
  global $tempname
  global $current_name
  global $double_files_copy;
  global $double_files
  global $doubles_count;
  global $simvidx
  global $simvcount;
  global $verbous;

  $tempname= "";

  while (!feof($fp_in)) {
    $count += 1;
    $buffer = fgets($fp_in,4096);
    $ret= parse_vmsg($buffer);
    if($verbous)
      echo "$buffer<BR>\n";
    if(!$vmsg_complete)
  continue;

    # Interpreter for particular message types    

    if($vmsg["header_subscr"] == "reply") {
      if($vmsg["state"]=="start") {
        $reply_mode= $vmsg[id]; 
     } else {
        if($vmsg[id]=="current_item") 
  break;
        $reply_mode= "off";
      }
    }
    if($vmsg["header_subscr"] == "stdout") {
      if($reply_mode != "off") {
        # collect complete lines
        $stdout_buffer.= $vmsg["text"];
        $nlpos= strpos($stdout_buffer,"\n");
        if($nlpos==false) 
  continue; 
        $gotline= substr($stdout_buffer, 0, $nlpos);
        $stdout_buffer= "";
        if($reply_mode == "current_item") {
          $msg= explode(" ",$gotline);
          $simvidx= $msg[0];
          $simvcount= $msg[1];
        }
      }
    }
    if($vmsg["header_subscr"] == "current_file_copy") {
      $tempname= $vmsg["file1"];
    }
    if($vmsg["header_subscr"] == "current_file") {
      $current_name= $vmsg["file1"];
    }
    if($vmsg["header_subscr"] == "double_files_copy") {
      $filecount= $vmsg["header_fieldcount"];
      for($i=1;$i<=$filecount;$i++) {
        $double_files_copy[$i-1]=  $vmsg["file$i"];
      }
    }
    if($vmsg["header_subscr"] == "double_files") {
      $doubles_count= $vmsg["header_fieldcount"];
      for($i=1;$i<=$doubles_count;$i++) {
        $double_files[$i-1]=  $vmsg["file$i"];
      }
    }

  }
  return 1;
}


# --- general HTML stuff and page layout


function doorman() {
# A very weak user authentication : user "me" , password "mypwd"
  global $PHP_AUTH_USER;
  global $PHP_AUTH_PW;

  if(!isset($PHP_AUTH_USER) ||
     ( $PHP_AUTH_USER != "me" || $PHP_AUTH_PW != "mypwd" ) ) {
    Header("WWW-Authenticate: Basic realm=\"PHP Testarea\"");
    Header("HTTP/1.0 401 Unauthorized");
    echo "<HTML><BODY>UNAUTHORIZED !!!</BODY></HTML>\n";
    exit;
  }
  return 1;
}


function write_header() {
  echo "<HTML>";
  echo "<TITLE>simv_per_query - an example</TITLE>\n";

  echo "<META HTTP-EQUIV=\"expires\" CONTENT=\"Thu, 01 Dec 1994 16:00:00 GMT\">";
  echo "<META HTTP-EQUIV=\"pragma\" CONTENT=\"no-cache\"\">\n";

  echo "<BODY BACKGROUND=\"slate.gif TEXT=\"#d0c0b0\"";
  echo " LINK=\"#c0c0ff\" VLINK=\"#c0c0ff\">\n";
  echo "<FONT SIZE=+2 COLOR=#d0c0b0>\n";
  return 1;
}


function write_verbous_cmd() {
# For debugging: write the actual sagent command and the raw reply lines into
# the HTML page.
  global $cmd;
  global $simvidx;
  global $simvcmd;

  echo "<BR><H2>query variables : <BR></H2>\n";
  echo "simvidx=$simvidx<BR>\n";
  echo "simvcmd=$simvcmd<BR>\n";
  echo "<BR><H2>executing : <BR></H2>\n";
  echo "$cmd<BR>\n";
  echo "<BR><H2>receiving : <BR></H2>\n";
  return 1;
}


function write_controls() {
# Writes the control links into the HTML document
# 
  global $simvidx
  global $simvcount;
  global $SCRIPT_NAME;

  echo "<CENTER><HR>\n";
  echo "<A HREF=\"" . $SCRIPT_NAME . "?simvidx=$simvidx&simvcmd=back\">&lt;&lt;&lt; back</A>";
  echo "| |";
  echo "<A HREF=\"" . $SCRIPT_NAME . "?simvidx=$simvidx&simvcmd=refresh\">refresh</A>";
  echo "| |";
  echo "<A HREF=\"" . $SCRIPT_NAME . "?simvidx=$simvidx&simvcmd=skip\">skip &gt;&gt;&gt;</A>";

  # offer an exponential scale of item numbers for jumping
  echo "<BR>\n<FONT SIZE=\"-1\">\n";
  for($step=2;$simvidx-$step>0 || $simvidx+$step<=$simvcount;$step *= 2){
    $i= $simvidx-$step;
    if($i>1)
      $downward = "<A HREF=\"" . $SCRIPT_NAME . "?simvidx=$i\">$i</A>_" 
                  . $downward;
    $i= $simvidx+$step;
    if($i<$simvcount)
      $upward = $upward
                . "_<A HREF=\"" . $SCRIPT_NAME . "?simvidx=$i\">$i</A>";
  }
  if($simvidx > 1)
    $downward = "<A HREF=\"" . $SCRIPT_NAME .  "?simvidx=1\">1</A>_|_"
                . $downward;
  if($simvidx < $simvcount)
    $upward = $upward . "_|_<A HREF=\"" . $SCRIPT_NAME 
              . "?simvidx=$simvcount\">$simvcount</A>";
  echo "$downward\n";
  echo "<FONT SIZE=\"+2\">$simvidx</FONT>\n";
  echo "$upward\n";
  echo "<BR></FONT></CENTER><HR>\n";
  return 1;
}


function write_doubles_note() {
# Write a notification about the duplicate files reported by simv
# over the current image.
  global $doubles_count;

  if($doubles_count>0) {
    echo "<CENTER>";
    echo "<IMG SRC=\"penguin.gif\"><BR>";
    echo "<H2>Note: &nbsp;&nbsp; $doubles_count possibly similar ";
    if($doubles_count==1) {
      echo "image ";
    } else {
      echo "images ";
    }
    echo "found (see below first one)";
    echo "<BR></H2><HR></CENTER>\n";
  }
  return 1;
}


function write_current_file() {
# Write the link to the temporary copy of the current image into the HTML page
  global $current_name;
  global $tempname;

  if(basename($current_name) != "" && $tempname != "") {
    echo "<CENTER>" . basename($current_name) . "<BR><BR>\n";
    echo "<IMG SRC=\"tmp/" . basename($tempname) . "\"><BR></CENTER>\n";
  }
  return 1;
}


function write_double_files() {
# Write the links to the temporary copies of duplicate files.
  global $doubles_count;
  global $double_files;
  global $double_files_copy;

  if($doubles_count>0) {
    echo "<CENTER><BR><BR><HR><HR>Possibly similar images :";
    echo "<BR></CENTER>\n";
    for($i=0;$i<$doubles_count;$i++){
      echo "<HR><LEFT>" . ($i+1) . "of $doubles_count</LEFT>\n";
      echo "<CENTER>$double_files[$i]<BR><BR>";
      echo "<IMG SRC=\"tmp/" . basename($double_files_copy[$i]) . "\">";
      echo "<BR></CENTER>\n";
    }
  }
  return 1;
}


function write_trailer() {
  echo "</BODY>\n";
  echo "</HTML>\n";
  return 1;
}


# --- main 

  doorman();

  write_header();

  make_cmd();
  if($verbous) 
    write_verbous_cmd();
  $fp_in = popen($cmd,"r");
  interpret_reply();
  pclose($fp_in);
  $fp_in= -1;

  write_controls();
  write_doubles_note();
  write_current_file();
  write_double_files();
  write_trailer();
?>
